var getTarget = function getTarget(value) {
  if (value instanceof HTMLElement) {
    return value;
  } else if (value === window.document) {
    return value.body;
  } else if (value instanceof Object) {
    return value.current;
  } else {
    return value;
  }
};

export var isDocumentElement = function isDocumentElement(target) {
  return target === window.document;
};

var calcBackToTopEngine = function calcBackToTopEngine(target, minHeight) {
  if (target === void 0) {
    target = window.document;
  }

  var element = getTarget(target);
  var isDocument = isDocumentElement(target);
  var scrollTop = isDocument ? window.scrollY : element.scrollTop;
  var clientHeight = isDocument ? window.innerHeight - (element.offsetHeight - element.clientHeight) : element.clientHeight;
  var offsetHeight = isDocument ? window.innerHeight : element.offsetHeight;
  var scrollHeight = isDocument ? element.scrollHeight : element.scrollHeight;
  var borderHeight = offsetHeight - clientHeight;
  var properties = {
    scrollTop: scrollTop,
    clientHeight: clientHeight,
    offsetHeight: offsetHeight,
    scrollHeight: scrollHeight,
    borderHeight: borderHeight,
    minHeight: minHeight
  };
  return [// if the scrolled area exceeds a quarter of its scrollable visible area, it might return true by default
  scrollTop > (minHeight !== undefined ? minHeight : clientHeight / 4), properties];
};

export default calcBackToTopEngine;