import _extends from "@babel/runtime/helpers/esm/extends";
import { useCallback, useEffect, forwardRef } from 'react';
import { createPortal } from 'react-dom';
import PropTypes from 'prop-types';
import cx from 'classnames';
import { getTarget } from '@s-ui/js/lib/react';
import useControlledState from '@s-ui/react-hooks/lib/useControlledState';
import calcBackToTopEngine, { isDocumentElement } from './calcBackToTopEngine';
import { SCROLL_BEHAVIOR, STYLES } from './settings';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
var BASE_CLASS = 'sui-AtomBackToTop';
var CLASS_ICON = BASE_CLASS + "-icon";
var CLASS_TEXT = BASE_CLASS + "-text";
var CLASS_SHOW = BASE_CLASS + "--show";
var CLASS_HIDE = BASE_CLASS + "--hide";
var CLASS_READY = BASE_CLASS + "--ready";
var AtomBackToTop = /*#__PURE__*/forwardRef(function (_ref, forwardedRef) {
  var IconTop = _ref.iconTop,
      isVisible = _ref.isVisible,
      minHeight = _ref.minHeight,
      onScroll = _ref.onScroll,
      onIsVisibleToggle = _ref.onIsVisibleToggle,
      _ref$refContainer = _ref.refContainer,
      refContainer = _ref$refContainer === void 0 ? window.document : _ref$refContainer,
      _ref$scrollBehavior = _ref.scrollBehavior,
      scrollBehavior = _ref$scrollBehavior === void 0 ? SCROLL_BEHAVIOR.SMOOTH : _ref$scrollBehavior,
      _ref$style = _ref.style,
      style = _ref$style === void 0 ? STYLES.DARK : _ref$style,
      textTop = _ref.textTop;

  var _useControlledState = useControlledState(isVisible),
      show = _useControlledState[0],
      setShow = _useControlledState[1];

  useEffect(function () {
    var container = getTarget(refContainer);

    var handleScroll = function handleScroll(event) {
      var _calcBackToTopEngine = calcBackToTopEngine(refContainer, minHeight),
          nextShow = _calcBackToTopEngine[0],
          properties = _calcBackToTopEngine[1];

      setShow(nextShow);
      typeof onScroll === 'function' && onScroll(event, _extends({}, properties, {
        show: show
      }));

      if (typeof onIsVisibleToggle === 'function' && show !== nextShow) {
        onIsVisibleToggle(event, _extends({}, properties, {
          show: nextShow
        }));
      }
    };

    container.removeEventListener('scroll', handleScroll);
    container.addEventListener('scroll', handleScroll);
    return function () {
      container.removeEventListener('scroll', handleScroll);
    };
  }, [minHeight, refContainer, show, setShow]);
  var scrollToTop = useCallback(function () {
    var container = getTarget(refContainer);
    var isDocument = isDocumentElement(container);
    (isDocument ? window : container).scrollTo({
      top: 0,
      behavior: scrollBehavior
    });
  }, [refContainer]);
  return /*#__PURE__*/createPortal( /*#__PURE__*/_jsxs("button", {
    title: "Back to top",
    className: cx(BASE_CLASS, BASE_CLASS + "--" + style, show !== null && CLASS_READY, show ? CLASS_SHOW : CLASS_HIDE),
    ref: forwardedRef,
    onClick: scrollToTop,
    children: [IconTop && /*#__PURE__*/_jsx("span", {
      className: CLASS_ICON,
      children: /*#__PURE__*/_jsx(IconTop, {})
    }), textTop && /*#__PURE__*/_jsx("span", {
      className: CLASS_TEXT,
      children: textTop
    })]
  }), refContainer === document ? refContainer.body : refContainer);
});
AtomBackToTop.displayName = 'AtomBackToTop';
export { STYLES as backToTopStyles };
export default AtomBackToTop;